/**
 * A single source of truth for all the classes used for decorations in Ixora.
 *  These are kept together here to simplify changing/adding classes later
 * and serve as a reference.
 *
 * Exports under this file don't need to follow any particular naming schema,
 * naming which can give an intuition on what the class is for is preferred.
 */
/** Classes for blockquote decorations. */
export declare const blockquote: {
    /** Blockquote widget */
    widget: string;
    /** Replace decoration for the quote mark */
    mark: string;
}, 
/** Classes for codeblock decorations. */
codeblock: {
    /** Codeblock widget */
    widget: string;
    /** First line of the codeblock widget */
    widgetBegin: string;
    /** Last line of the codeblock widget */
    widgetEnd: string;
}, 
/** Classes for heading decorations. */
heading: {
    /** Heading decoration class */
    heading: string;
    /** Heading levels (h1, h2, etc) */
    level: (level: number) => string;
    /** Heading slug */
    slug: (slug: string) => string;
}, 
/** Classes for link (URL) widgets. */
link: {
    /** URL widget */
    widget: string;
}, 
/** Classes for list widgets. */
list: {
    /** List bullet */
    bullet: string;
    /** List task checkbox */
    taskCheckbox: string;
    /** Task list item with checkbox checked */
    taskChecked: string;
}, 
/** Classes for image widgets. */
image: {
    /** Image preview */
    widget: string;
};
