var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import {
  Decoration,
  EditorView,
  ViewPlugin,
  WidgetType
} from "@codemirror/view";
import {
  iterateTreeInVisibleRanges,
  editorLines,
  isCursorInRange,
  checkRangeSubset
} from "../util";
import { blockquote as classes } from "../classes";
const quoteMarkRE = /^(\s*>+)/gm;
class BlockQuoteBorderWidget extends WidgetType {
  toDOM() {
    const dom = document.createElement("span");
    dom.classList.add(classes.mark);
    return dom;
  }
}
class BlockQuotePlugin {
  constructor(view) {
    __publicField(this, "decorations");
    this.decorations = this.styleBlockquote(view);
  }
  update(update) {
    if (update.docChanged || update.viewportChanged || update.selectionSet) {
      this.decorations = this.styleBlockquote(update.view);
    }
  }
  styleBlockquote(view) {
    const widgets = [];
    iterateTreeInVisibleRanges(view, {
      enter: ({ name, from, to }) => {
        if (name !== "Blockquote")
          return;
        const lines = editorLines(view, from, to);
        lines.forEach((line) => {
          const lineDec = Decoration.line({
            class: classes.widget
          });
          widgets.push(lineDec.range(line.from));
        });
        if (lines.every((line) => !isCursorInRange(view, [line.from, line.to]))) {
          const marks = Array.from(view.state.sliceDoc(from, to).matchAll(quoteMarkRE)).map((x) => from + x.index).map((i) => Decoration.replace({
            widget: new BlockQuoteBorderWidget()
          }).range(i, i + 1));
          lines.forEach((line) => {
            if (!marks.some((mark) => checkRangeSubset([line.from, line.to], [mark.from, mark.to])))
              marks.push(Decoration.widget({
                widget: new BlockQuoteBorderWidget()
              }).range(line.from));
          });
          widgets.push(...marks);
        }
      }
    });
    return Decoration.set(widgets, true);
  }
}
const blockQuotePlugin = ViewPlugin.fromClass(BlockQuotePlugin, {
  decorations: (v) => v.decorations
});
const baseTheme = EditorView.baseTheme({
  ["." + classes.mark]: {
    "border-left": "4px solid #ccc"
  },
  ["." + classes.widget]: {
    color: "#555"
  }
});
function blockquote() {
  return [blockQuotePlugin, baseTheme];
}
export {
  blockquote
};
//# sourceMappingURL=blockquote.js.map
