var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import {
  Decoration,
  ViewPlugin
} from "@codemirror/view";
import {
  checkRangeOverlap,
  invisibleDecoration,
  isCursorInRange,
  iterateTreeInVisibleRanges
} from "../util";
const typesWithMarks = [
  "Emphasis",
  "StrongEmphasis",
  "InlineCode",
  "Strikethrough"
];
const markTypes = ["EmphasisMark", "CodeMark", "StrikethroughMark"];
class HideMarkPlugin {
  constructor(view) {
    __publicField(this, "decorations");
    this.decorations = this.compute(view);
  }
  update(update) {
    if (update.docChanged || update.viewportChanged || update.selectionSet)
      this.decorations = this.compute(update.view);
  }
  compute(view) {
    const widgets = [];
    let parentRange;
    iterateTreeInVisibleRanges(view, {
      enter: ({ type, from, to, node }) => {
        if (typesWithMarks.includes(type.name)) {
          if (parentRange && checkRangeOverlap([from, to], parentRange))
            return;
          else
            parentRange = [from, to];
          if (isCursorInRange(view, [from, to]))
            return;
          const innerTree = node.toTree();
          innerTree.iterate({
            enter({ type: type2, from: markFrom, to: markTo }) {
              if (!markTypes.includes(type2.name))
                return;
              widgets.push(invisibleDecoration.range(from + markFrom, from + markTo));
            }
          });
        }
      }
    });
    return Decoration.set(widgets, true);
  }
}
const hideMarks = () => [
  ViewPlugin.fromClass(HideMarkPlugin, {
    decorations: (v) => v.decorations
  })
];
export {
  hideMarks,
  markTypes,
  typesWithMarks
};
//# sourceMappingURL=hide-mark.js.map
