var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import {
  Decoration,
  EditorView,
  ViewPlugin,
  WidgetType
} from "@codemirror/view";
import { isCursorInRange, iterateTreeInVisibleRanges } from "../util";
import { list as classes } from "../classes";
const bulletListMarkerRE = /^[-+*]/;
const lists = () => [listBulletPlugin, taskListPlugin, baseTheme];
class ListBulletPlugin {
  constructor(view) {
    __publicField(this, "decorations", Decoration.none);
    this.decorations = this.decorateLists(view);
  }
  update(update) {
    if (update.docChanged || update.viewportChanged || update.selectionSet)
      this.decorations = this.decorateLists(update.view);
  }
  decorateLists(view) {
    const widgets = [];
    iterateTreeInVisibleRanges(view, {
      enter: ({ type, from, to }) => {
        if (isCursorInRange(view, [from, to]))
          return;
        if (type.name === "ListMark") {
          const listMark = view.state.sliceDoc(from, to);
          if (bulletListMarkerRE.test(listMark)) {
            const dec = Decoration.replace({
              widget: new ListBulletWidget(listMark)
            });
            widgets.push(dec.range(from, to));
          }
        }
      }
    });
    return Decoration.set(widgets, true);
  }
}
const listBulletPlugin = ViewPlugin.fromClass(ListBulletPlugin, {
  decorations: (v) => v.decorations
});
class ListBulletWidget extends WidgetType {
  constructor(bullet) {
    super();
    this.bullet = bullet;
  }
  toDOM() {
    const listBullet = document.createElement("span");
    listBullet.textContent = this.bullet;
    listBullet.className = "cm-list-bullet";
    return listBullet;
  }
}
class TaskListsPlugin {
  constructor(view) {
    __publicField(this, "decorations", Decoration.none);
    this.decorations = this.addCheckboxes(view);
  }
  update(update) {
    if (update.docChanged || update.viewportChanged || update.selectionSet)
      this.decorations = this.addCheckboxes(update.view);
  }
  addCheckboxes(view) {
    const widgets = [];
    iterateTreeInVisibleRanges(view, {
      enter: this.iterateTree(view, widgets)
    });
    return Decoration.set(widgets, true);
  }
  iterateTree(view, widgets) {
    return ({ type, from, to, node }) => {
      if (type.name !== "Task")
        return;
      let checked = false;
      node.toTree().iterate({
        enter: (ref) => iterateInner(ref.type, ref.from, ref.to)
      });
      if (checked)
        widgets.push(Decoration.mark({
          tagName: "span",
          class: "cm-task-checked"
        }).range(from, to));
      function iterateInner(type2, nfrom, nto) {
        if (type2.name !== "TaskMarker")
          return;
        if (isCursorInRange(view, [from + nfrom, from + nto]))
          return;
        const checkbox = view.state.sliceDoc(from + nfrom, from + nto);
        if ("xX".includes(checkbox[1]))
          checked = true;
        const dec = Decoration.replace({
          widget: new CheckboxWidget(checked, from + nfrom + 1)
        });
        widgets.push(dec.range(from + nfrom, from + nto));
      }
    };
  }
}
class CheckboxWidget extends WidgetType {
  constructor(checked, pos) {
    super();
    this.checked = checked;
    this.pos = pos;
  }
  toDOM(view) {
    const wrap = document.createElement("span");
    wrap.classList.add(classes.taskCheckbox);
    const checkbox = document.createElement("input");
    checkbox.type = "checkbox";
    checkbox.checked = this.checked;
    checkbox.addEventListener("click", ({ target }) => {
      const change = {
        from: this.pos,
        to: this.pos + 1,
        insert: this.checked ? " " : "x"
      };
      view.dispatch({ changes: change });
      this.checked = !this.checked;
      target.checked = this.checked;
    });
    wrap.appendChild(checkbox);
    return wrap;
  }
}
const taskListPlugin = ViewPlugin.fromClass(TaskListsPlugin, {
  decorations: (v) => v.decorations
});
const baseTheme = EditorView.baseTheme({
  ["." + classes.bullet]: {
    position: "relative",
    visibility: "hidden"
  },
  ["." + classes.taskChecked]: {
    textDecoration: "line-through !important"
  },
  ["." + classes.bullet + ":after"]: {
    visibility: "visible",
    position: "absolute",
    top: 0,
    left: 0,
    content: "'\\2022'"
  }
});
export {
  lists
};
//# sourceMappingURL=list.js.map
