import { syntaxTree } from "@codemirror/language";
import {
  StateField,
  StateEffect
} from "@codemirror/state";
import {
  Decoration,
  WidgetType,
  EditorView
} from "@codemirror/view";
import { image as classes } from "../classes";
var WidgetState = /* @__PURE__ */ ((WidgetState2) => {
  WidgetState2[WidgetState2["INITIAL"] = 0] = "INITIAL";
  WidgetState2[WidgetState2["LOADED"] = 1] = "LOADED";
  return WidgetState2;
})(WidgetState || {});
const imageLoadedEffect = StateEffect.define();
const imagePreview = StateField.define({
  create(state) {
    const images = extractImages(state);
    const decorations = images.map((img) => Decoration.widget({
      widget: new ImagePreviewWidget(img, 0 /* INITIAL */),
      src: img.src
    }).range(img.to));
    return Decoration.set(decorations, true);
  },
  update(value, tx) {
    const loadedImages = tx.effects.filter((effect) => effect.is(imageLoadedEffect));
    if (tx.docChanged || loadedImages.length > 0) {
      const images = extractImages(tx.state);
      const decorations = images.map((img) => {
        const hasImageLoaded = Boolean(loadedImages.find((effect) => effect.value.src === img.src));
        return Decoration.widget({
          widget: new ImagePreviewWidget(img, hasImageLoaded ? 1 /* LOADED */ : 0 /* INITIAL */),
          block: hasImageLoaded ? true : false,
          src: img.src,
          side: 1
        }).range(img.to);
      });
      return Decoration.set(decorations, true);
    } else {
      return value.map(tx.changes);
    }
  },
  provide(field) {
    return EditorView.decorations.from(field);
  }
});
const imageTextRE = /(?:!\[)(.*?)(?:\])/;
function extractImages(state) {
  const imageUrls = [];
  syntaxTree(state).iterate({
    enter: ({ name, node, from, to }) => {
      if (name !== "Image")
        return;
      const alt = state.sliceDoc(from, to).match(imageTextRE).pop();
      const urlNode = node.getChild("URL");
      if (urlNode) {
        const url = state.sliceDoc(urlNode.from, urlNode.to);
        imageUrls.push({ src: url, from, to, alt });
      }
    }
  });
  return imageUrls;
}
class ImagePreviewWidget extends WidgetType {
  constructor(info, state) {
    super();
    this.info = info;
    this.state = state;
  }
  toDOM(view) {
    const img = new Image();
    img.classList.add(classes.widget);
    img.src = this.info.src;
    img.alt = this.info.alt;
    img.addEventListener("load", () => {
      const tx = {};
      if (this.state === 0 /* INITIAL */) {
        tx.effects = [imageLoadedEffect.of(this.info)];
      }
      view.dispatch(tx);
    });
    if (this.state === 1 /* LOADED */)
      return img;
    else
      return new Image();
  }
  eq(widget) {
    return JSON.stringify(widget.info) === JSON.stringify(this.info) && widget.state === this.state;
  }
}
export {
  WidgetState,
  imageLoadedEffect,
  imagePreview
};
//# sourceMappingURL=image.js.map
