import type { SyntaxNodeRef } from '@lezer/common';
import { Decoration, EditorView } from '@codemirror/view';
/**
 * Check if two ranges overlap
 * Based on the visual diagram on https://stackoverflow.com/a/25369187
 * @param range1 - Range 1
 * @param range2 - Range 2
 * @returns True if the ranges overlap
 */
export declare function checkRangeOverlap(range1: [number, number], range2: [number, number]): boolean;
/**
 * Check if a range is inside another range
 * @param parent - Parent (bigger) range
 * @param child - Child (smaller) range
 * @returns True if child is inside parent
 */
export declare function checkRangeSubset(parent: [number, number], child: [number, number]): boolean;
/**
 * Check if any of the editor cursors is in the given range
 * @param view - Editor view
 * @param range - Range to check
 * @returns True if the cursor is in the range
 */
export declare function isCursorInRange(view: EditorView, range: [number, number]): boolean;
/**
 * Iterate over the syntax tree in the visible ranges of the document
 * @param view - Editor view
 * @param iterateFns - Object with `enter` and `leave` iterate function
 */
export declare function iterateTreeInVisibleRanges(view: EditorView, iterateFns: {
    enter(node: SyntaxNodeRef): boolean | void;
    leave?(node: SyntaxNodeRef): void;
}): void;
/**
 * Decoration to simply hide anything.
 */
export declare const invisibleDecoration: Decoration;
/**
 * Returns the lines of the editor that are in the given range and not folded.
 * This function is of use when you need to get the lines of a particular
 * block node and add line decorations to each line of it.
 *
 * @param view - Editor view
 * @param from - Start of the range
 * @param to - End of the range
 * @returns A list of line blocks that are in the range
 */
export declare function editorLines(view: EditorView, from: number, to: number): import("@codemirror/view").BlockInfo[];
/**
 * Class containing methods to generate slugs from heading contents.
 */
export declare class Slugger {
    /** Occurrences for each slug. */
    private occurences;
    /**
     * Generate a slug from the given content.
     * @param text - Content to generate the slug from
     * @returns the slug
     */
    slug(text: string): string;
}
